#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2011 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Timetabling app integration.
"""

from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.interface import implements, directlyProvides
from zope.intid.interfaces import IIntIds
from zope.component import adapts, getUtility, getMultiAdapter
from zope.traversing.browser.interfaces import IAbsoluteURL
from zope.publisher.browser import BrowserView
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.schema.vocabulary import getVocabularyRegistry
from zope.traversing.browser.absoluteurl import absoluteURL
from zc.table.interfaces import ISortableColumn

import schooltool.skin.flourish.page
from schooltool.app.browser.app import ActiveSchoolYearContentMixin
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.utils import TitledContainerItemVocabulary
from schooltool.common.inlinept import InlineViewPageTemplate
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.skin.containers import ContainerView
from schooltool.skin import flourish
from schooltool.table import table
from schooltool.term.interfaces import ITerm
from schooltool.timetable.interfaces import IHaveSchedule
from schooltool.timetable.interfaces import IHaveTimetables
from schooltool.timetable.interfaces import IScheduleContainer
from schooltool.timetable.interfaces import ITimetableContainer

from schooltool.common import SchoolToolMessage as _


class ScheduleContainerAbsoluteURLAdapter(BrowserView):

    adapts(IScheduleContainer, IBrowserRequest)
    implements(IAbsoluteURL)

    traversal_name = 'schedule'

    def __str__(self):
        container_id = int(self.context.__name__)
        int_ids = getUtility(IIntIds)
        container = int_ids.getObject(container_id)
        url = str(getMultiAdapter((container, self.request), name='absolute_url'))
        return '%s/%s' % (url, self.traversal_name)

    __call__ = __str__


class TimetableContainerAbsoluteURLAdapter(ScheduleContainerAbsoluteURLAdapter):
    adapts(ITimetableContainer, IBrowserRequest)

    traversal_name = 'timetables'


# XXX: the view is not working yet, default selection broken
# XXX: timetable deletion view not implemented
class TimetableContainerView(ContainerView):
    """TimetableContainer view."""

    index_title = _("School Timetables")

    def update(self):
        if 'UPDATE_SUBMIT' in self.request:
            default = self.context.get(self.request['ttschema'])
            self.context.default = default
        return ''


class TimetableContainerTableFormatter(table.SchoolToolTableFormatter):

    def columns(self):
        title = table.LocaleAwareGetterColumn(
            name='title',
            title=_(u"Title"),
            getter=lambda i, f: i.title,
            subsort=True)
        starts = table.GetterColumn(
            name='starts',
            title=_(u"Starts"),
            getter=lambda i, f: i.first,
            subsort=True)
        ends = table.GetterColumn(
            name='ends',
            title=_(u"Ends"),
            getter=lambda i, f: i.last,
            subsort=True)
        directlyProvides(title, ISortableColumn)
        directlyProvides(starts, ISortableColumn)
        directlyProvides(ends, ISortableColumn)
        return [title, starts, ends]

    def sortOn(self):
        return (('title', False), ("starts", False), ("ends", False),)


class FlourishTimetableContainerView(table.TableContainerView):

    def getColumnsAfter(self):
        delete = table.ImageInputColumn(
            'delete', title=_('Delete'),
            library='schooltool.skin.flourish',
            image='remove-icon.png',
            id_getter=table.simple_form_key)
        return [delete]

    def update(self):
        super(FlourishTimetableContainerView, self).update()

        # XXX: deletion without confirmation is quite dangerous
        delete = [
            key for key, item in self.container.items()
            if "delete.%s" % table.simple_form_key(item) in self.request]
        for key in delete:
            del self.container[key]
        if delete:
            self.request.response.redirect(
                absoluteURL(self.context, self.request))


class FlourishTimetableContainerLinks(flourish.page.RefineLinksViewlet):
    """demographics fields add links viewlet."""


def getActivityVocabulary(object=None):
    if object is None:
        object = ISchoolToolApplication(None)
    vr = getVocabularyRegistry()
    vocabulary = vr.get(object, 'schooltool.timetable.activityvocbulary')
    return vocabulary


class TimetableVocabulary(TitledContainerItemVocabulary):
    @property
    def container(self):
        owner = IHaveSchedule(self.context)
        return ITimetableContainer(ISchoolYear(ITerm(owner)), {})


def timetableVocabularyFactory():
    return TimetableVocabulary


class SchoolTimetablesTertiaryNavigation(flourish.page.Content,
                                         flourish.page.TertiaryNavigationManager,
                                         ActiveSchoolYearContentMixin):

    template = InlineViewPageTemplate("""
        <ul tal:attributes="class view/list_class">
          <li tal:repeat="item view/items"
              tal:attributes="class item/class">
              <a tal:attributes="href item/url"
                 tal:content="item/schoolyear/@@title" />
          </li>
        </ul>
    """)

    @property
    def items(self):
        result = []
        active = self.schoolyear
        schoolyears = active.__parent__ if active is not None else {}
        for schoolyear in schoolyears.values():
            url = '%s/%s?schoolyear_id=%s' % (
                absoluteURL(self.context, self.request),
                'timetables',
                schoolyear.__name__)
            result.append({
                    'class': schoolyear.first == active.first and 'active' or None,
                    'url': url,
                    'schoolyear': schoolyear,
                    })
        return result


class FlourishTimetablesView(table.TableContainerView,
                             ActiveSchoolYearContentMixin):

    content_template = ViewPageTemplateFile('templates/f_timetables.pt')

    @property
    def title(self):
        schoolyear = self.schoolyear
        return _('Timetables for ${schoolyear}',
                 mapping={'schoolyear': schoolyear.title})

    @property
    def container(self):
        return ITimetableContainer(self.schoolyear)


class TimetableAddLinks(flourish.page.RefineLinksViewlet):
    """Manager for Add links in FlourishTimetablesView"""


class TimetablesLinkViewlet(flourish.page.LinkViewlet,
                            ActiveSchoolYearContentMixin):

    def __init__(self, context, request, *args, **kw):
        self.request = request
        super(TimetablesLinkViewlet, self).__init__(
            self.actualContext(context), request, *args, **kw)

    def actualContext(self, context):
        return ITimetableContainer(self.schoolyear)


class TimetableDoneLink(ActiveSchoolYearContentMixin):
    template = InlineViewPageTemplate('''
        <h3 class="done-link">
          <a tal:attributes="href view/url" tal:content="view/title" />
        </h3>
    ''')

    title = _('Done')

    @property
    def schoolyear(self):
        container = ITimetableContainer(self.context)
        return ISchoolYear(IHaveTimetables(container))

    @property
    def url(self):
        app = ISchoolToolApplication(None)
        return self.url_with_schoolyear_id(app, view_name='timetables')


class FlourishManageTimetablesOverview(flourish.page.Content,
                                       ActiveSchoolYearContentMixin):

    body_template = ViewPageTemplateFile('templates/f_manage_timetables_overview.pt')

    @property
    def timetables(self):
        timetables = ITimetableContainer(self.schoolyear, None)
        if timetables is not None:
            return sorted(timetables.values(), key=lambda t:t.first,
                          reverse=True)

    def timetables_url(self):
        return self.url_with_schoolyear_id(self.context,
                                           view_name='timetables')



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Relationship interfaces
"""

from zope.interface import Interface, Attribute


class IRelationshipProperty(Interface):
    """A property tied to a relationship."""

    rel_type = Attribute("""Relationship type""")
    my_role = Attribute("""Role of the object with this property""")
    other_role = Attribute("""Role of other objects in this relationship""")
    this = Attribute("""The object that this property is bound to.""")
    relationships = Attribute(
        """A list of RelationshipInfo helpers for all related objects.""")

    def __nonzero__():
        """Are there any related objects?

        Equivalent to bool(list(self)), but can be implemented more
        efficiently.
        """

    def __len__():
        """Return the number of related objects."""

    def __iter__():
        """Iterate over all related objects."""

    def add(other, extra_info=None):
        """Establish a relationship with `other`."""

    def remove(other):
        """Unlink a relationship with `other`."""


class IRelationshipInfo(Interface):
    """Non-persistent helper to access relationship extra information."""

    source = Attribute("""Source object of the relationship.""")
    target = Attribute("""Target object of the relationship.""")
    extra_info = Attribute("""Extra information that was passed to `relate`.""")


class IRelationshipLink(Interface):
    """One half of a relationship.

    When a relationship between `a` and `b` is established, two links are
    created and placed into link sets of `a` and `b` respectively.
    """

    rel_type = Attribute("""Relationship type.""")
    target = Attribute("""The other member of the relationship.""")
    role = Attribute("""Role of `target`.""")
    my_role = Attribute("""Role of the object that has this link.""")
    extra_info = Attribute("""Extra information that was passed to `relate`.

        Be careful to keep extra info in sync on both links of the
        relationship.
        """)


class IRelationshipLinks(Interface):
    """A set of relationship links.

    Objects that can be used in relationships are those that have an adapter to
    IRelationshipLinks.

    See also `IRelationshipLink`.
    """

    def __iter__():
        """Iterate over all links."""

    def add(link):
        """Add a new link."""

    def remove(link):
        """Remove a link.

        Raises ValueError if link is not in the set.
        """

    def clear():
        """Remove all links."""

    def find(my_role, target, role, rel_type):
        """Find a link with matching attributes.

        Raises ValueError if a matching link is not found.
        """

    def __getitem__(id):
        """Return the link with a given id."""

    def get(key, default=None):
        """Get a link for a key

        The default is returned if there is no value for the key.
        """

    def getTargetsByRole(role, rel_type=None):
        """Return all objects related to this object with a given role."""


class IRelationshipEvent(Interface):
    """Common attributes for relationship events."""

    rel_type = Attribute("""Relationship type.""")
    participant1 = Attribute("""One of the participants.""")
    role1 = Attribute("""Role of `participant1`.""")
    participant2 = Attribute("""One of the participants.""")
    role2 = Attribute("""Role of `participant2`.""")
    extra_info = Attribute("""Extra info, as passed to relate().""")

    def __getitem__(role):
        """Return the participant with a given role.

        This does not work well with "peer-to-peer" relationships, that is,
        when role1 == role2.
        """

    def match(schema):
        """Return an accessor object if this event matches the schema."""


class IBeforeRelationshipEvent(IRelationshipEvent):
    """A relationship is about to be established.

    You can register subscribers for this event to implement constraints.
    By convention, subscribers raise InvalidRelationship or a subclass
    when a relationship does not meet all the constraints.
    """


class IRelationshipAddedEvent(IRelationshipEvent):
    """A relationship has been established."""


class IBeforeRemovingRelationshipEvent(IRelationshipEvent):
    """A relationship is about to be broken.

    You can register subscribers for this event to implement constraints
    (by raising exceptions).
    """


class IRelationshipRemovedEvent(IRelationshipEvent):
    """A relationship has been broken."""


class InvalidRelationship(Exception):
    """Invalid relationship."""


class DuplicateRelationship(InvalidRelationship):
    """Relationship already exists"""


class NoSuchRelationship(Exception):
    """Relationship does not exist"""


class IRelationshipSchema(Interface):
    """Relationship schema."""

    roles = Attribute("""Roles of members of the relationship.""")
    rel_type = Attribute("""Type of the relationship.""")

    def __call__(**parties):
        """Establish a relationship."""

    def unlink(**parties):
        """Break a relationship."""

    def query(**party):
        """Retrieve relationship targets."""



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
SchoolTool skin.
"""
from zope.interface import directlyProvidedBy
from zope.interface import directlyProvides
from zope.publisher.interfaces.browser import IBrowserRequest

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.skin import ISchoolToolSkin


class IDevModeLayer(IBrowserRequest):
    """SchoolTool devmode layer."""


class ISchoolToolDevModeSkin(IDevModeLayer, ISchoolToolSkin):
    """The SchoolTool devmode skin"""


def schoolToolTraverseSubscriber(event):
    """A subscriber to BeforeTraverseEvent.

    Adds DevMode layer to the request.
    """
    if (ISchoolToolApplication.providedBy(event.object) and
        IBrowserRequest.providedBy(event.request)):
        directlyProvides(event.request,
                         directlyProvidedBy(event.request) + IDevModeLayer)



import kombu
import os

try:
    import celery.app.abstract
    assert celery.app.abstract  # silence pyflakes
    CELERY3 = False
except:
    CELERY3 = True

if CELERY3:
    CELERY_QUEUES = (
        kombu.Queue('default', kombu.Exchange('default'), routing_key='default'),
    #    kombu.Queue('import',  kombu.Exchange('zodb'),   routing_key='zodb.import'),
        kombu.Queue('zodb.report', kombu.Exchange('default'),   routing_key='zodb.report'),
    )
else:
    CELERY_QUEUES = {
        "default": {
            "exchange": "default",
            "binding_key": "default",
            },
        "zodb.report": {
            "exchange": "default",
            "binding_key": "zodb.report",
            },
        "zodb.import": {
            "exchange": "default",
            "binding_key": "zodb.import",
            },
        }

CELERY_DEFAULT_QUEUE = 'default'
CELERY_DEFAULT_EXCHANGE_TYPE = 'direct'
CELERY_DEFAULT_ROUTING_KEY = 'default'

_REDIS_HOST = str(os.environ.get('REDIS_HOST', "localhost"))
_REDIS_PORT = int(os.environ.get('REDIS_PORT', 7079))
_REDIS_DB = 1

if CELERY3:
    CELERY_RESULT_BACKEND = "redis://%s:%d/%d" % (
        _REDIS_HOST, _REDIS_PORT, _REDIS_DB)
else:
    CELERY_RESULT_BACKEND = "redis"
    CELERY_REDIS_HOST = _REDIS_HOST
    CELERY_REDIS_PORT = _REDIS_PORT
    CELERY_REDIS_DB = _REDIS_DB

_BROKER_REDIS_DB = 0

BROKER_URL = "redis://%s:%d/%d" % (
    _REDIS_HOST, _REDIS_PORT, _BROKER_REDIS_DB)

CELERY_ENABLE_UTC = True

CELERY_IMPORTS = ("schooltool.task.tasks", )

#CELERYBEAT_OPTS="--schedule=/home/justas/src/schooltool/flourish_celery/instance/var/celerybeat-schedule"
#CELERYBEAT_SCHEDULE = {}

CELERY_STORE_ERRORS_EVEN_IF_IGNORED = True
CELERY_SERIALIZER = 'json'
CELERY_ACCEPT_CONTENT = ['json', 'pickle']

SCHOOLTOOL_CONFIG = os.environ.get('SCHOOLTOOL_CONF')
SCHOOLTOOL_RETRY_DB_CONFLICTS = 3
